@@ -1,6 +1,4 @@
 """scvi-tools Model classes for contrastive-VI."""
 from .contrastive_vi import ContrastiveVIModel as ContrastiveVI
 
-__all__ = [
-    "ContrastiveVI"
-]
+__all__ = ["ContrastiveVI"]
